/*---------------------------------------------------------------------------+
 |  reg_norm.c                                                               |
 |                                                                           |
 | Copyright (C) 1992,1993,1994,1995,1997,1999                               |
 |                       W. Metzenthen, 22 Parker St, Ormond, Vic 3163,      |
 |                       Australia.  E-mail billm@melbpc.org.au              |
 |                                                                           |
 | Normalize the value in a FPU_REG.                                         |
 |                                                                           |
 |                                                                           |
 |    Return value is the tag of the answer, or-ed with FPU_Exception if     |
 |    one was raised, or -1 on internal error.                               |
 |                                                                           |
 +---------------------------------------------------------------------------*/

#include "fpu_emu.h"



int FPU_normalize(FPU_REG *x)
{

  if ( ! (x->sigh & 0x80000000) )
    {
      if ( x->sigh == 0 )
	{
	  if ( x->sigl == 0 )
	    {
	      x->exp = 0;
	      return TAG_Zero;
	    }
	  x->sigh = x->sigl;
	  x->sigl = 0;
	  x->exp -= 32;
	}
      while ( !(x->sigh & 0x80000000) )
	{
	  x->sigh <<= 1;
	  if ( x->sigl & 0x80000000 )
	    x->sigh |= 1;
	  x->sigl <<= 1;
	  x->exp --;
	}
    }

  if ( x->exp >= EXP_OVER )
    {
      x->exp += EXTENDED_Ebias;
      return arith_overflow(x);
    }

  if ( x->exp <= EXP_UNDER )
    {
      x->exp += EXTENDED_Ebias;
      return arith_underflow(x);
    }

  x->exp += EXTENDED_Ebias;

  return TAG_Valid;
}


int FPU_normalize_nuo(FPU_REG *x)
{

  if ( ! (x->sigh & 0x80000000) )
    {
      if ( x->sigh == 0 )
	{
	  if ( x->sigl == 0 )
	    {
	      x->exp = EXP_UNDER;
	      return TAG_Zero;
	    }
	  x->sigh = x->sigl;
	  x->sigl = 0;
	  x->exp -= 32;
	}
      while ( !(x->sigh & 0x80000000) )
	{
	  x->sigh <<= 1;
	  if ( x->sigl & 0x80000000 )
	    x->sigh |= 1;
	  x->sigl <<= 1;
	  x->exp --;
	}
    }

  return TAG_Valid;
}
